/*
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    David Bonneau and Obeo - initial API and implementation
 */

package fr.obeo.acceleo.gen.debug.ui.model;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IAdapterFactory;
import org.eclipse.debug.ui.actions.IRunToLineTarget;
import org.eclipse.debug.ui.actions.IToggleBreakpointsTarget;
import org.eclipse.ui.texteditor.ITextEditor;

/**
 * Creates a toggle breakpoint adapter.
 * 
 * @author David Bonneau
 */
public class AcceleoBreakpointAdapterFactory implements IAdapterFactory {

	/* (non-Javadoc) */
	public Object getAdapter(Object adaptableObject, Class adapterType) {
		if (adaptableObject instanceof ITextEditor) {
			ITextEditor editorPart = (ITextEditor) adaptableObject;
			IResource resource = (IResource) editorPart.getEditorInput().getAdapter(IResource.class);
			if (resource != null) {
				String extension = resource.getFileExtension();
				if (extension != null && extension.equals(getExtension())) {
					if (adapterType.equals(IToggleBreakpointsTarget.class)) {
						return createAcceleoLineBreakpointAdapter();
					}
					if (adapterType.equals(IRunToLineTarget.class)) {
						return new AcceleoRunToLineAdapter();
					}
				}
			}
		}
		return null;
	}

	/* (non-Javadoc) */
	public Class[] getAdapterList() {
		return new Class[] { IToggleBreakpointsTarget.class };
	}

	/**
	 * @return the file extension
	 */
	protected String getExtension() {
		return "mt"; //$NON-NLS-1$
	}

	/**
	 * @return the breakpoint adapter
	 */
	protected IToggleBreakpointsTarget createAcceleoLineBreakpointAdapter() {
		return new AcceleoLineBreakpointAdapter();
	}

}
